using NiceAction.Character.FSM;
using NiceAction.Character.FSM.Struct;
using NiceAction.Character.Data;
using UnityEngine;

namespace NiceAction.Character // TODO : 맨 상위 폴더의 넴스페이스는 플젝이름으로, 그 이후 부터는 폴더 이름 따라가기
{
    public class CharacterBase : MonoBehaviour
    {
        [SerializeField] private CharacterController m_CharacterController;
        [SerializeField] private Animator m_Animator;
        [SerializeField] private CharacterPhysicsData m_PhysicsData;
        [SerializeField] private SpriteRenderer m_SpriteRenderer;
        
        private CharacterInput m_Input;
        private CharacterStateMachine m_StateMachine;
        private CharacterMovement m_Movement;
        private CharacterAnimator m_Anim;
        private CharacterPhysics m_Physics;

        public void Start()
        {
            // TODO : 각 요소마다 순서대로 이니셜 라이징 시키기.
            m_StateMachine = new CharacterStateMachine();
            m_Input = new CharacterInput(m_StateMachine);
            m_Movement = new CharacterMovement(m_CharacterController, m_StateMachine, m_PhysicsData, m_SpriteRenderer);
            m_Anim = new CharacterAnimator(m_Animator, m_StateMachine);
            m_Physics = new CharacterPhysics(this.transform, m_CharacterController, m_PhysicsData);
        }

        public void Update()
        {
            var deltaTime = Time.deltaTime;
            m_Physics.OnUpdate();
            m_Input.OnUpdate();

            m_StateMachine.OnUpdate(CreateCondition());
            m_Movement.OnUpdate(deltaTime);
            m_Anim.OnUpdate();
        }

        private CharacterCondition CreateCondition()
        {
            return new CharacterCondition(m_Input.m_HorizontalInput, m_Physics.isOnGround, m_Movement.moveSpeed);
        }
    }
}
