using System.Collections.Generic;
using NiceAction.Character.FSM.Command;
using NiceAction.Character.FSM.State;
using NiceAction.Character.FSM.Struct;

namespace NiceAction.Character.FSM
{
    public enum CharacterStateType
    {
        Jump,
        Idle,
        Run,
        OnAir,
    }

    public class CharacterStateMachine
    {
        public CharacterStateType state => m_CurrState.state;
        public bool isStateChanged => m_CurrState != m_PrevState;

        public BaseState m_CurrState { get; private set; }
        public BaseState m_PrevState { get; private set; }

        private List<ICharacterCommand> m_CommandsList;

        //init
        public CharacterStateMachine()
        {
            m_CommandsList = new List<ICharacterCommand>();
            m_CurrState = new IdleState();
            m_PrevState = m_CurrState;
        }

        public void OnUpdate(CharacterCondition condition)
        {
            m_PrevState = m_CurrState;

            m_CurrState.OnUpdate(condition);
            m_CurrState = m_CurrState.nextState ?? m_CurrState;

            DoCommand();
            m_CommandsList.Clear();
        }

        public void AddCommand(ICharacterCommand characterCommand)
        {
            m_CommandsList.Add(characterCommand);
        }

        private void DoCommand()
        {
            foreach (var command in m_CommandsList)
            {
                switch (command)
                {
                    case JumpCharacterCommand:
                        JumpTransition();
                        break;
                    case BeIdleCommand:
                        BeIdleTransition();
                        break;
                    case MoveCharacterCommand:
                        MovingTransition();
                        break;
                }
            }
        }

        private void JumpTransition()
        {
            if (this.state != CharacterStateType.OnAir)
            {
                m_CurrState = new JumpState();
            }
        }

        private void OnGroundTransition(bool isMoving)
        {
            if (state != CharacterStateType.Idle)
            {
                if (isMoving)
                {
                    m_CurrState = new RunState();
                }
                else
                {
                    m_CurrState = new IdleState();
                }
            }
        }

        private void MovingTransition()
        {
            if (state == CharacterStateType.Idle)
            {
                m_CurrState = new RunState();
            }
        }

        private void BeIdleTransition()
        {
            if (m_CurrState is not IdleState)
                m_CurrState = new IdleState();
        }

    }
}
