using UnityEngine;

namespace NiceAction
{
    public abstract class SingletonMonoBehaviour<T> : MonoBehaviour where T : MonoBehaviour
    {
        static T myInstance = null;
        public static T Instance
        {
            get
            {
                if (myInstance == null)
                {
                    Debug.LogWarning($"SingleTonMono Find Try");
                    myInstance = FindObjectOfType<T>();

                    if (myInstance == null)
                    {
                        Debug.LogWarning($"SingleTonMono Create Try");
                        GameObject obj = new GameObject();
                        obj.name = typeof(T).FullName + "_Singleton";
                        DontDestroyOnLoad(obj);
                        myInstance = obj.AddComponent<T>();
                    }
                }
                return myInstance;
            }
        }
        private void Awake()
        {
            DontDestroyOnLoad(gameObject);
            if (myInstance == null)
            {
                myInstance = this as T;
            }
            OnAwake();
        }

        protected virtual void OnAwake() { }
    }
}
